@@ -4,7 +4,7 @@ A Python library for interacting asynchronously with the Spotify API
 
 __title__ = 'asyncspotify'
 __author__ = 'RUNIE'
-__version__ = '0.11.4'
+__version__ = '0.11.8'
 __license__ = 'MIT'
 
 import logging
@@ -12,12 +12,15 @@ import logging
 from . import utils
 from .album import FullAlbum, SimpleAlbum
 from .artist import FullArtist, SimpleArtist
+from .audioanalysis import AudioAnalysis
+from .audiofeatures import AudioFeatures
 from .client import Client
 from .device import Device
 from .exceptions import *
+from .http import Route
 from .image import Image
-from .oauth import AuthorizationCodeFlow, ClientCredentialsFlow, EasyCodeFlow
-from .object import Object
+from .oauth import AuthorizationCodeFlow, ClientCredentialsFlow, EasyAuthorizationCodeFlow
+from .object import SpotifyObject
 from .playing import CurrentlyPlaying, CurrentlyPlayingContext
 from .playlist import FullPlaylist, SimplePlaylist
 from .scope import Scope
