@@ -1,15 +1,15 @@
 import logging
 from datetime import timedelta
-from typing import List, Optional
+from typing import List
 
 from .album import FullAlbum, SimpleAlbum
 from .artist import FullArtist, SimpleArtist
+from .audioanalysis import AudioAnalysis
 from .audiofeatures import AudioFeatures
 from .device import Device
-from .exceptions import NotFound, SpotifyException
 from .http import HTTP
-from .oauth.flows import Authenticator
-from .object import Object
+from .oauth.flows import Authenticator, RefreshableFlowMixin
+from .object import SpotifyObject
 from .pager import CursorBasedPaging, Pager, SearchPager
 from .playing import CurrentlyPlaying, CurrentlyPlayingContext
 from .playlist import FullPlaylist, SimplePlaylist
@@ -21,12 +21,16 @@ log = logging.getLogger(__name__)
 
 
 def get_id(obj):
-	if isinstance(obj, Object):
+	if isinstance(obj, SpotifyObject):
 		return obj.id
 	else:
 		return obj
 
 
+def clamp(limit, minimum):
+	return None if limit is None else min(max(limit, 1), minimum)
+
+
 class Client:
 	'''
 	Client interface for the API.
@@ -59,15 +63,25 @@ class Client:
 
 	async def authorize(self):
 		'''Tell the authenticator to authorize this client.'''
+
 		await self.auth.authorize()
 
 	async def refresh(self):
 		'''Tell the authenticator to refresh this client, if applicable.'''
+
+		if not isinstance(self.auth, RefreshableFlowMixin):
+			raise TypeError('Authenticator does not support refreshing.')
+
 		await self.auth.refresh()
 
 	async def close(self):
 		'''Close this client session.'''
-		await self.http.close_session()
+
+		await self.auth.close()
+		await self.http.close()
+
+	def _ensure_market(self, market):
+		return self.auth.market if market is None else market
 
 	async def get_player(self, **kwargs) -> CurrentlyPlayingContext:
 		'''
@@ -191,16 +205,14 @@ class Client:
 
 		await self.http.player_shuffle(state, get_id(device))
 
-	async def search(self, *types, q, market=None, limit=None, offset=None, include_external=None) -> dict:
+	async def search(self, *types, q, limit=20, market=None, offset=None, include_external=None) -> dict:
 		'''
 		Searches for tracks, artists, albums and/or playlists.
 
-		https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2
-
 		:param types: One or more of the strings ``track``, ``album``, ``artist``, ``playlist`` or the class equivalents.
 		:param str q: The search query. See Spotifys' query construction guide `here. <https://developer.spotify.com/documentation/web-api/reference/search/search/>`_
-		:param market: ISO-3166_ country code or the string ``from_token``.
 		:param int limit: How many results of each type to return.
+		:param market: ISO-3166-1_ country code or the string ``from_token``.
 		:param offset: Where to start the pagination.
 		:param include_external: If this is equal to ``audio``, the specified the response will include any relevant audio content that is hosted externally.
 
@@ -219,7 +231,7 @@ class Client:
 		data = await self.http.search(
 			q, ','.join(actual_types),
 			market=market,
-			limit=limit,
+			limit=clamp(limit, 50),
 			offset=offset,
 			include_external=include_external
 		)
@@ -248,49 +260,52 @@ class Client:
 
 		return results
 
-	async def search_tracks(self, q, market=None, limit=None, offset=None, include_external=None) -> List[SimpleTrack]:
+	async def search_tracks(self, q, limit=20, market=None, offset=None, include_external=None) -> List[SimpleTrack]:
 		'''
 		Alias for ``Client.search('track', ...)``
 
 		:return: List[:class:`SimpleTrack`]
 		'''
 
-		results = await self.search('track', q=q, market=market, limit=limit, offset=offset,
-			include_external=include_external)
+		results = await self.search(
+			'track', q=q, market=market, limit=limit, offset=offset, include_external=include_external
+		)
 		return results['tracks']
 
-	async def search_artists(self, q, market=None, limit=None, offset=None, include_external=None) -> List[FullArtist]:
+	async def search_artists(self, q, limit=20, market=None, offset=None, include_external=None) -> List[FullArtist]:
 		'''
 		Alias for ``Client.search('artist, ...)``
 
 		:return: List[:class:`FullArtist`]
 		'''
 
-		results = await self.search('artist', q=q, market=market, limit=limit, offset=offset,
-			include_external=include_external)
+		results = await self.search(
+			'artist', q=q, market=market, limit=limit, offset=offset, include_external=include_external
+		)
 		return results['artists']
 
-	async def search_albums(self, q, market=None, limit=None, offset=None, include_external=None) -> List[SimpleAlbum]:
+	async def search_albums(self, q, limit=20, market=None, offset=None, include_external=None) -> List[SimpleAlbum]:
 		'''
 		Alias for ``Client.search('album', ...)``
 
 		:return: List[:class:`SimpleAlbum`]
 		'''
 
-		results = await self.search('album', q=q, market=market, limit=limit, offset=offset,
-			include_external=include_external)
+		results = await self.search(
+			'album', q=q, market=market, limit=limit, offset=offset, include_external=include_external
+		)
 		return results['albums']
 
-	async def search_playlists(self, q, market=None, limit=None, offset=None, include_external=None) -> List[
-		SimplePlaylist]:
+	async def search_playlists(self, q, limit=20, market=None, offset=None, include_external=None) -> List[SimplePlaylist]:
 		'''
 		Alias for ``Client.search('playlist', ...)``
 
 		:return: List[:class:`SimplePlaylist`]
 		'''
 
-		results = await self.search('playlist', q=q, market=market, limit=limit, offset=offset,
-			include_external=include_external)
+		results = await self.search(
+			'playlist', q=q, market=market, limit=limit, offset=offset, include_external=include_external
+		)
 		return results['playlists']
 
 	async def search_track(self, q=None) -> SimpleTrack:
@@ -343,7 +358,7 @@ class Client:
 		data = await self.http.get_me()
 		return PrivateUser(self, data)
 
-	async def get_me_top_tracks(self, limit=None, offset=None, time_range=None) -> List[SimpleTrack]:
+	async def get_me_top_tracks(self, limit=20, offset=None, time_range=None) -> List[SimpleTrack]:
 		'''
 		Gets the top tracks of the current user.
 
@@ -361,10 +376,7 @@ class Client:
 		:return: List[:class:`SimpleTrack`]
 		'''
 
-		if limit > 50:
-			raise SpotifyException('Limit must be less or equal to 50.')
-
-		data = await self.http.get_me_top_tracks(limit=limit, offset=offset, time_range=time_range)
+		data = await self.http.get_me_top_tracks(limit=clamp(limit, 50), offset=offset, time_range=time_range)
 
 		tracks = []
 		for track_obj in data['items']:
@@ -388,10 +400,7 @@ class Client:
 		:return: List[:class:`SimpleArtist`]
 		'''
 
-		if limit > 50:
-			raise SpotifyException('Limit must be less or equal to 50.')
-
-		data = await self.http.get_me_top_artists(limit=limit, offset=offset, time_range=time_range)
+		data = await self.http.get_me_top_artists(limit=clamp(limit, 50), offset=offset, time_range=time_range)
 
 		artists = []
 		for artist_obj in data['items']:
@@ -399,7 +408,7 @@ class Client:
 
 		return artists
 
-	async def get_user(self, user_id) -> Optional[PublicUser]:
+	async def get_user(self, user_id) -> PublicUser:
 		'''
 		Get a user.
 
@@ -407,10 +416,7 @@ class Client:
 		:return: A :class:`PublicUser` instance.
 		'''
 
-		try:
-			data = await self.http.get_user(user_id)
-		except NotFound:
-			return None
+		data = await self.http.get_user(user_id)
 
 		return PublicUser(self, data)
 
@@ -481,7 +487,7 @@ class Client:
 		for chunk in subslice(uris, 100):
 			await self.http.playlist_add_tracks(playlist, uris=chunk, position=position)
 
-	async def get_playlist(self, playlist_id) -> Optional[FullPlaylist]:
+	async def get_playlist(self, playlist_id) -> FullPlaylist:
 		'''
 		Get a pre-existing playlist.
 
@@ -489,13 +495,9 @@ class Client:
 		:return: :class:`FullPlaylist` instance.
 		'''
 
-		try:
-			data = await self.http.get_playlist(playlist_id)
-		except NotFound:
-			return None
+		data = await self.http.get_playlist(playlist_id)
 
 		playlist = FullPlaylist(self, data)
-		await playlist._fill_tracks(PlaylistTrack, Pager(self.http, data.pop('tracks')))
 
 		return playlist
 
@@ -536,16 +538,11 @@ class Client:
 			if playlist_obj is None:
 				playlists.append(None)
 			else:
-				playlist = SimplePlaylist(self, playlist_obj)
-				await playlist._fill_tracks(
-					PlaylistTrack,
-					Pager(self.http, await self.http.get_playlist_tracks(playlist.id))
-				)
-				playlists.append(playlist)
+				playlists.append(SimplePlaylist(self, playlist_obj))
 
 		return playlists
 
-	async def get_track(self, track_id) -> Optional[FullTrack]:
+	async def get_track(self, track_id) -> FullTrack:
 		'''
 		Get a track.
 
@@ -553,11 +550,7 @@ class Client:
 		:return: :class:`FullTrack` instance.
 		'''
 
-		try:
-			data = await self.http.get_track(track_id)
-		except NotFound:
-			return None
-
+		data = await self.http.get_track(track_id)
 		return FullTrack(self, data)
 
 	async def get_tracks(self, *track_ids) -> List[FullTrack]:
@@ -581,9 +574,9 @@ class Client:
 
 		return tracks
 
-	async def get_audio_features(self, track) -> Optional[AudioFeatures]:
+	async def get_audio_features(self, track) -> AudioFeatures:
 		'''
-		Get 'Audio Features' of a track.
+		Get Audio Features of a track.
 
 		:param track: :class:`Track` instance or Spotify ID.
 		:return: :class:`AudioFeatures`
@@ -591,14 +584,25 @@ class Client:
 
 		track = get_id(track)
 
-		try:
-			data = await self.http.get_audio_features(track)
-		except NotFound:
-			return None
+		data = await self.http.get_audio_features(track)
 
 		return AudioFeatures(self, data)
 
-	async def get_artist(self, artist_id) -> Optional[FullArtist]:
+	async def get_audio_analysis(self, track) -> AudioAnalysis:
+		'''
+		Get Audio Analysis of a track.
+
+		:param track: :class:`Track` instance or Spotify ID.
+		:return: :class:`AudioAnalysis`
+		'''
+
+		track = get_id(track)
+
+		data = await self.http.get_audio_analysis(track)
+
+		return AudioAnalysis(self, data)
+
+	async def get_artist(self, artist_id) -> FullArtist:
 		'''
 		Get an artist.
 
@@ -606,17 +610,18 @@ class Client:
 		:return: :class:`FullArtist` instance.
 		'''
 
-		try:
-			data = await self.http.get_artist(artist_id)
-		except NotFound:
-			return None
+		data = await self.http.get_artist(artist_id)
 
 		return FullArtist(self, data)
 
-	async def get_artist_albums(self, artist_id, include_groups=None, country=None, limit=None, offset=None) -> List[
-		SimpleAlbum]:
+	async def get_artist_albums(self, artist_id, limit=20, include_groups=None, country=None, offset=None) -> List[SimpleAlbum]:
 		'''
-		Get an artist's albums
+		Get an artist's albums.
+
+		.. note::
+		   This endpoint does *not* return the track objects for each album. If you need those, you have to
+		   fetch them manually afterwards.
+
 		:param str artist_id: Spotify ID of artist.
 		:param int limit: How many albums to return.
 		:param kwargs: other query params for this method
@@ -625,12 +630,9 @@ class Client:
 
 		albums = []
 
-		try:
-			data = await self.http.get_artist_albums(
-				artist_id, include_groups=include_groups, country=country, limit=max(limit, 50), offset=offset
-			)
-		except NotFound:
-			return albums
+		data = await self.http.get_artist_albums(
+			artist_id, include_groups=include_groups, country=country, limit=clamp(limit, 50), offset=offset
+		)
 
 		async for album_obj in Pager(self.http, data, limit):
 			albums.append(SimpleAlbum(self, album_obj))
@@ -658,16 +660,17 @@ class Client:
 
 		return artists
 
-	async def get_artist_top_tracks(self, artist, market='from_token') -> List[FullTrack]:
+	async def get_artist_top_tracks(self, artist, market=None) -> List[FullTrack]:
 		'''
 		Returns the top tracks for an artist.
 
 		:param artist: :class:`Artist` instance or Spotify ID.
-		:param market: Market to find tracks for.
+		:param market: ISO-3166-1_ country code. Leave blank to let the library auto-resolve this.
 		:return: A list of maximum 10 :class:`FullTrack` instances.
 		'''
 
 		artist_id = get_id(artist)
+		market = self._ensure_market(market)
 
 		data = await self.http.get_artist_top_tracks(artist_id, market=market)
 
@@ -697,22 +700,18 @@ class Client:
 
 		return artists
 
-	async def get_album(self, album_id, **kwargs) -> Optional[FullAlbum]:
+	async def get_album(self, album_id, market=None) -> FullAlbum:
 		'''
 		Get an album.
 
 		:param str album_id: Spotify ID of album.
-		:param kwargs: other query params for this method
+		:param market: ISO-3166-1_ country code.
 		:return: :class:`FullAlbum` instance.
 		'''
 
-		try:
-			data = await self.http.get_album(album_id, **kwargs)
-		except NotFound:
-			return None
+		data = await self.http.get_album(album_id, market=market)
 
 		album = FullAlbum(self, data)
-		await album._fill_tracks(SimpleTrack, Pager(self.http, data['tracks']))
 
 		return album
 
@@ -721,7 +720,7 @@ class Client:
 		Get several albums.
 
 		:param str album_ids: Spotify ID of album.
-		:param kwargs: other query params for this method
+		:param market: ISO-3166-1_ country code.
 		:return: List[:class:`FullAlbum`]
 		'''
 
@@ -734,45 +733,44 @@ class Client:
 				if album_obj is None:
 					albums.append(None)
 				else:
-					album = FullAlbum(self, album_obj)
-					await album._fill_tracks(SimpleTrack, Pager(self.http, album_obj['tracks']))
-					albums.append(album)
+					albums.append(FullAlbum(self, album_obj))
 
 		return albums
 
-	async def get_album_tracks(self, album, **kwargs) -> List[SimpleTrack]:
+	async def get_album_tracks(self, album, limit=20, offset=None, market=None) -> List[SimpleTrack]:
 		'''
 		Get tracks from an album.
 
 		:param album: :class:`Album` or Spotify ID of album.
-		:param kwargs: other query params for this method
+		:param limit: How many tracks to fetch.
+		:param offset: What pagination offset to start from.
+		:param market: ISO-3166-1_ country code.
 		:return: List[:class:`SimpleTrack`]
 		'''
 
 		album = get_id(album)
 
-		data = await self.http.get_album_tracks(album, **kwargs)
+		# assuming the limit is 50 for now
+		data = await self.http.get_album_tracks(album, limit=clamp(limit, 50), offset=offset, market=market)
 
 		tracks = []
 
-		async for track_obj in Pager(self.http, data):
+		async for track_obj in Pager(self.http, data, limit):
 			tracks.append(SimpleTrack(self, track_obj))
 
 		return tracks
 
-	async def get_followed_artists(self, type='artist', limit=None, after=None) -> List[SimpleArtist]:
+	async def get_followed_artists(self, limit=20, after=None) -> List[SimpleArtist]:
 		'''
 		Get user's followed artists
 
-		:param str type: The ID type: currently only artist is supported.
 		:param int limit: The maximum number of items to return. Default - infinity
+		:param after: What artist ID to start the fetching from.
 		:return: List[:class:`SimpleArtist`]
 		'''
 
-		if type != 'artist':
-			raise SpotifyException('Currently only artist is supported.')
-
-		data = await self.http.get_followed_artists(type=type, limit=limit, after=after)
+		# assuming the limit is 50 for now
+		data = await self.http.get_followed_artists(type='artist', limit=clamp(limit, 50), after=after)
 
 		artists = []
 
@@ -791,5 +789,8 @@ class Client:
 		:param after: The last artist ID retrieved from the previous request.
 		'''
 
+		# not sure what this endpoint is doing, it doesn't take a limit at least. will look at it some other time
+		raise NotImplementedError
+
 		for chunk in subslice(ids, 50):
-			await self.http.following(type=type, ids=','.join(get_id(obj) for obj in chunk), limit=limit, after=after)
+			await self.http.following(type=type, ids=','.join(get_id(obj) for obj in chunk), limit=clamp(limit, 50), after=after)
