@@ -41,11 +41,10 @@ class HTTP:
 		self.session = ClientSession(loop=loop or asyncio.get_event_loop())
 		self.lock = Lock()
 
-	async def close_session(self):
+	async def close(self):
 		await self.session.close()
 
 	async def request(self, route, data=None, json=None, headers=None, authorize=True):
-
 		if authorize:
 			auth_header = self.client.auth.header
 			if auth_header is None:
@@ -243,6 +242,10 @@ class HTTP:
 		r = Route('GET', 'audio-features/{0}'.format(track_id))
 		return await self.request(r)
 
+	async def get_audio_analysis(self, track_id):
+		r = Route('GET', 'audio-analysis/{0}'.format(track_id))
+		return await self.request(r)
+
 	async def get_artist(self, artist_id):
 		r = Route('GET', 'artists/{0}'.format(artist_id))
 		return await self.request(r)
