@@ -6,15 +6,16 @@ log = logging.getLogger(__name__)
 
 
 class Pager:
-	def __init__(self, http, pager_object, stop_after=None):
+	def __init__(self, http, pager_object, limit=None):
 		self.http = http
 		self.pos = 0
-		self.stop_after = stop_after
+		self.self_limit = limit
 		self.set_next(pager_object)
 
 	def set_next(self, obj):
 		self.total = obj['total']
 		self.next = obj['next']
+		self.previous = obj['previous']
 		self.items = obj['items']
 		self.limit = obj['limit']
 		self.offset = obj['offset']
@@ -28,11 +29,11 @@ class Pager:
 		return self
 
 	async def __anext__(self):
-		# stop if we hit the pager total
+		# stop if we hit the pager total or the specified pager limit
 		if self.pos >= self.total:
 			raise StopAsyncIteration
 
-		if self.stop_after is not None and self.pos >= self.stop_after:
+		if self.self_limit is not None and self.pos >= self.self_limit:
 			raise StopAsyncIteration
 
 		# get the next page if we're exhausted this one
@@ -45,9 +46,9 @@ class Pager:
 
 
 class SearchPager(Pager):
-	def __init__(self, http, obj, type, stop_after=None):
+	def __init__(self, http, obj, type, limit=None):
 		self.type = type
-		super().__init__(http, obj, stop_after)
+		super().__init__(http, obj, limit)
 
 	def set_next(self, obj):
 		obj = obj[self.type]
